#!/usr/bin/nawk -f

# Jan Gorodkin
# Center for Biological Sequence Analysis
# The Technical University of Denmark
# B206, DK-2800 Lyngby
# Denmark
# gorodkin@cbs.dtu.dk

# example to execute
# ./align2symvec.awk -v Itype=2 -v startpos=1 -v aadistfile="aadist1"
# A file align.info will be generated. This file contains all the relevant
# information concerning information contents of the logo.




BEGIN{}
NR==1{

 # Checking for correct assignments:
 if(Itype!=1 && Itype!=2)
 {
   Itype=2;
   print "\nWarning variable Itype should be 1 or 2.\n";
   print "Itype has been sat equal 2.\n";
 }

 k=1;  #seq cuonter

}
{
   if(NR==1 && $1==">") seqlength = length($2);

   # read sequences
   if($1==">") 
   { 
     for(i=1;i<seqlength+1;i++) seq[k,i]=substr($2,i,1);
     k++;
   }

}
END{

   # Reading nt dist. for each position. Or use the previous line.
   # The latter means that if you only have one line in the aadistfile
   # it corresponds to having only one backgorund dist for the whole
   # alignment.
   pos=1;
   while( (getline < aadistfile) > 0 )
   {
      if(NF==20)
      {
        for(j=1;j<21;j++) p[j,pos]=$j;
        p[21,pos]=1;
        probline=$0;
      }

      #check if probs add to one
      tmp=0.0;
      for(j=1;j<21;j++) tmp+=int(10000.0*p[j,pos]);
      if(tmp!=10000)
      {
          print "Your a priori probabilities do not add to one!  (exit)";
          print tmp/10000"   "probline;
          exit;
      }
     pos++;
   }
   close(aadistfile);

   if(pos<seqlength+1)
     for(i=pos;i<seqlength+1;i++)
     {
        for(j=1;j<21;j++) p[j,i]=p[j,i-1];
        p[21,i]=1;
     }



###################################################################
   # SEQUENCE INFORMATION of each position
   print "pos     tot inf."  > "align.info";

   for(i=1;i<seqlength+1;i++)
   {      
      for(l=1;l<k;l++)
      {
         if(seq[l,i]=="A") q[1,i]++;
         else if(seq[l,i]=="C") q[2,i]++;
         else if(seq[l,i]=="D") q[3,i]++;
         else if(seq[l,i]=="E") q[4,i]++;
         else if(seq[l,i]=="F") q[5,i]++;
         else if(seq[l,i]=="G") q[6,i]++;
         else if(seq[l,i]=="H") q[7,i]++;
         else if(seq[l,i]=="I") q[8,i]++;
         else if(seq[l,i]=="K") q[9,i]++;
         else if(seq[l,i]=="L") q[10,i]++;
         else if(seq[l,i]=="M") q[11,i]++;
         else if(seq[l,i]=="N") q[12,i]++;
         else if(seq[l,i]=="P") q[13,i]++;
         else if(seq[l,i]=="Q") q[14,i]++;
         else if(seq[l,i]=="R") q[15,i]++;
         else if(seq[l,i]=="S") q[16,i]++;
         else if(seq[l,i]=="T") q[17,i]++;
         else if(seq[l,i]=="V") q[18,i]++;
         else if(seq[l,i]=="W") q[19,i]++;
         else if(seq[l,i]=="Y") q[20,i]++;
         else if(seq[l,i]=="-") q[21,i]++;
         else { print "not allowed symbol:  "seq[l,i];  exit; }
      }

      for(j=1;j<22;j++)
      {
         q[j,i] = q[j,i]/(k-1);
         if(q[j,i] > 0) IC[j,i] = q[j,i] * log ( q[j,i]/p[j,i] ) /log(2);
         else IC[j,i] = 0;
      }
     
      Ipos[i]=0;
      for(j=1;j<22;j++) Ipos[i] += IC[j,i];

      ICtot += Ipos[i];

      print i+1-startpos"          "Ipos[i] >> "align.info";


   }

###################################################################





   #print beginning of symvec file
   print "* alpro 1.64";
   print "*.";
   print "*.";
   print "* PROTEIN STRUCTURE ALIGNMENT";   
   print "* position, samples, information, variance";   
   print "21 number of symbols";

   # calculating the total information content
   for(ii=startpos;ii<seqlength+startpos;ii++)
   {
      i=ii-startpos+1;

      Jnum=0;
      Jtot = Ipos[i];
      
      if(Itype==1)
      {   
        for(n=1;n<22;n++)
        {
          tmp=int(100000.0*q[n,i]*Ipos[i]);
	  #if(q[n,i]<p[n,i]) Inum[n] = -tmp;
          #else Inum[n] = tmp;
          Inum[n] = tmp;
          tmp2 = sqrt(tmp*tmp);
          Jnum = Jnum+tmp2;
        }
      }
      else if(Itype==2)
      {
        NORM=0;
        for(n=1;n<22;n++)  NORM += q[n,i]/p[n,i];

        for(n=1;n<22;n++)
        {
          tmp=int(100000.0 * ( (q[n,i]/p[n,i])/NORM) * Ipos[i]);
          Inum[n] = tmp;
          tmp2 = sqrt(tmp*tmp);
          Jnum = Jnum+tmp2;
        }
      }


      letter[1]="A";
      letter[2]="C";
      letter[3]="D";
      letter[4]="E";
      letter[5]="F";
      letter[6]="G";
      letter[7]="H";
      letter[8]="I";
      letter[9]="K";
      letter[10]="L";
      letter[11]="M";
      letter[12]="N";
      letter[13]="P";
      letter[14]="Q";
      letter[15]="R";
      letter[16]="S";
      letter[17]="T";
      letter[18]="V";
      letter[19]="W";
      letter[20]="Y";
      letter[21]="-";
      

      for(n=1;n<22;n++) Tnum[n]=sqrt(Inum[n]*Inum[n]);

      for(n=1;n<22;n++)
      {
         for(m=n+1;m<22;m++)
	 {
            if(Tnum[n]>Tnum[m])
	    {
               Ttmp=Tnum[m];
               Tnum[m]=Tnum[n];
               Tnum[n]=Ttmp;

               Itmp=Inum[m];
               Inum[m]=Inum[n];
               Inum[n]=Itmp;

               ltmp=letter[m];
               letter[m]=letter[n];
               letter[n]=ltmp;
            }
         }
      }

      sizecheck=0;
      for(n=1;n<22;n++) if(Inum[n]>0) sizecheck=1;
      if(sizecheck==0) Jtot=0.0;

      printf("%2d  %5d   %f   0\n", ii, Jnum, Jtot);

      for(n=1;n<22;n++) printf("%c   %d\n", letter[n], Inum[n]);


      Jseq+=Jtot;

   }

    print "Information content for the whole alignment:  "Jseq >> "align.info";

}
























